### R code from vignette source 'sprofiling.Rnw'
### Encoding: ISO8859-1

###################################################
### code chunk number 1: sprofiling.Rnw:183-193
###################################################
cleanpackages <- function()
{
	# make sure we do not get inherited methods or garbage here.
	try(detach("package:sna"), silent=TRUE)
	try(detach("package:igraph"), silent=TRUE)
	try(detach("package:network"), silent=TRUE)
	try(detach("package:graph"), silent=TRUE)
	try(detach("package:Rgraphviz"), silent=TRUE)
	try(detach("package:graph"), silent=TRUE)
}


###################################################
### code chunk number 2: sprofiling.Rnw:196-197
###################################################
library(sprof)


###################################################
### code chunk number 3: sprofiling.Rnw:203-205
###################################################
message("switching  options(warn=1) -- immediate warning on")
options(warn=1)


###################################################
### code chunk number 4: sprofiling.Rnw:208-209
###################################################
#options(error = recover)


###################################################
### code chunk number 5: sprofiling.Rnw:213-215
###################################################
options(width = 72)
options(digits = 6)


###################################################
### code chunk number 6: sprofiling.Rnw:218-231
###################################################
strx <- function(x, 
	max.level=2, vec.len=3, 
	nchar.max=40, 
	list.len=12,
	width=70, strict.width="wrap",...)
{
	cat(paste("##strx:",deparse(substitute(x)),"\n"))
	str(x, max.level=max.level, 
		vec.len=vec.len, 
		nchar.max=nchar.max, 
		list.len=list.len,
		width=width, strict.width=strict.width,...)
}


###################################################
### code chunk number 7: sprofiling.Rnw:238-284
###################################################
xcutrows <- function(df,  cut, margin=5)
{
	if (!is.data.frame(df)) return(df)
	nrow <- nrow(df)
	# cut a range if it is not empty. 
	# Quiet noop else. 
	# Does not cut single lines.
	 cutrng <- function(cutfrom,cutto, cutname="<cut>"){
	 	if (cutfrom<cutto){
		df[cutfrom,] <- NA
		if (!is.null(rownames(df))) rownames(df)[cutfrom] <- cutname
		if (!is.null(df$name)) df$name[cutfrom] <- ""
		
		cutfrom <- cutfrom+1
		df[-(cutfrom:cutto),]
		}#if
	}
	
	if (!missing(cut))  {
		if (length(cut) ==2) 
		{df <- cutrng(cut[1],cut[2])
			return(df)
		} else {
		warning("cut ignored. Cut range must be a vector of length 2")
			return(df)}
	}
	if (length(margin)==0) return(df)
	if (length(margin)==1) margin <-  c(margin, margin)
	if (length(margin)==2) {
		cut <- c(margin[1]+1,nrow-margin[2])
		df <-cutrng(cutfrom=cut[1],cutto=cut[2]); 
		return(df)
	}
	
	#len=3: margin low,  center, margin up
	delta <- (nrow-sum(margin))  %/% 2
	if (delta<2) return(df)
	c1 <- margin[1]+1
	c2 <- c1 + delta -1
	c4 <- nrow-  margin[3]
	c3 <- c4 - delta 
	df <- cutrng(c3,c4, cutname="<cut hi>")
	df <- cutrng(c1,c2, cutname="<cut low>")
	return(df)
}



###################################################
### code chunk number 8: sprofiling.Rnw:289-325
###################################################
library(xtable)
prxt <- function (x, digits=2, cut=TRUE, caption=NULL, 
	label=NULL, zero.print=NULL, print.results=TRUE,...)  {
	
	if (cut) {margin <- 10
	if (nrow(x)> 2*margin+3) x <-xcutrows(x, margin=margin)}

#special sanitising for xtable
xr <- rownames(x)
#for (i in (1:length(xr))) {xr[i] <- sub(xr[i],"\\[","$[$", fixed=TRUE)}
 #xr <- paste("$",xr,"$")
xr <- gsub("[","{[}",xr, fixed=TRUE)
xr <- gsub("_","\\_",xr, fixed=TRUE)
xr <- gsub("^","\\^",xr, fixed=TRUE)
rownames(x)<- xr
	
	pr <- print(
		xtable(x, digits=digits, caption=caption, 
				label=label, ...), 
		floating=FALSE, 
		tabular.environment="longtable", 
		caption.placement="top",
		zero.print = ".",
		NA.string="\\vdots", 
		print.results=FALSE)
#		NA.string="",  #NA.string="\\vdots", 
		
		pr <- gsub( "$\backslash$vdots","\vdots",x=pr, fixed=TRUE)
		
		if(!is.null(zero.print)) 
			pr <- gsub( " 0 ",zero.print, x=pr, fixed=TRUE)

			
		if (print.results) cat(pr)
		invisible(pr)
	}


###################################################
### code chunk number 9: plotviz
###################################################
plotviz <- function(x, layout="dot", main=NULL, sub=NULL,...)
{
	xid <- deparse(substitute(x))
	xsubid <- NULL
	class1 <- NULL
	if (inherits(x,"sprof")) {
		class1 <- paste0("class orig: ", paste(class(x), collapse=" "))
		xsubid <- x$info$id
		sub <- paste0(sub, " ", x$info$id)
		x <- as(adjacency(x), "graphNEL")
	}

	if (!is.null(sub)) sub <- as.character(sub)
	if (is.null(main)) 
		main <-paste0("plotviz( ", xid ,", ", layout, " )\n", xsubid) else
		main=paste0(main, "\n plotviz( ", xid ,", ", layout, " )\n", xsubid)
	
	if (inherits(x,"Ragraph")) {
	plot(x=x,  y=layout,
			cex.main=1.2, 
		main=main,sub=sub,...)
	} else {
		plot(x=x,  y=layout,
		attrs=list(
			node=list(cex=4, fontsize=40,  shape="ellipse")), 
		cex.main=1.2, 
		main=main,sub=sub,...)
	}
		
#	title(sub = paste0(sub, " ", as.character(class(x)))	)
		title(sub = "xx try sub xx")
	legend("topleft", 
	
	
		legend=c(class1,
			paste0("class: ", paste(class(x), collapse=" ")), 
			paste0("layout: ",layout)),
		bg="#FFFFE040",
		seg.len=0
		)#"lightyellow" =#FFFFE0
}



###################################################
### code chunk number 10: RegressionExpl
###################################################
n <- 10000
x <- runif(n)
err <- rnorm(n)
y <- 2+ 3 * x + err
reg0data <- data.frame(x=x, y=y, err=err)
rm(x,y,err)



###################################################
### code chunk number 11: Rprof01
###################################################
profinterval <- 0.001
simruns <- 100

Rprof(filename="RprofsRegressionExpl.out", interval = profinterval)
	for (i in 1:simruns) xxx<- summary(lm(y~x, data=reg0data))
Rprof(NULL)


###################################################
### code chunk number 12: Rprof01mem
###################################################
Rprof(filename="RprofsRegressionExplMem.out", 
	interval = profinterval, memory.profiling = TRUE)
	for (i in 1:simruns) xxx<- summary(lm(y~x, data=reg0data))
Rprof(NULL)
sumRprofRegressionExplMem <- 
readRprof("RprofsRegressionExplMem.out")



###################################################
### code chunk number 13: Rprof01gc
###################################################
Rprof(filename="RprofsRegressionExplGC.out",
	interval = profinterval, gc.profiling = FALSE)
	for (i in 1:(simruns*10)) xxx<- summary(lm(y~x, data=reg0data))
Rprof(NULL)
sumRprofRegressionExplGC <- 
readRprof("RprofsRegressionExplGC.out")


###################################################
### code chunk number 14: sprofiling.Rnw:473-476
###################################################
sumRprofRegressionExpl <- summaryRprof("RprofsRegressionExpl01.out")
#str(profile_nodes_rle, max.level=2, vec.len=3, nchar.max=40, list.len=6)
strx(sumRprofRegressionExpl)


###################################################
### code chunk number 15: tab:prSRREbs
###################################################
prxt(sumRprofRegressionExpl$by.self,
	caption="summaryRprof result: by.self as final stack entry, all records", 
	label="tab:prSRREbs")


###################################################
### code chunk number 16: Rdbarplots
###################################################
oldpar <- par(mfrow=c(3,2))
x <- runif(32)
barplot_s(x)
barplot_s(x, decreasing=FALSE)
barplot_s(x, lowtrim=0.1, hightrim=0.8)
barplot_s(x, lowtrim=0.1, hightrim=0.8, colfun="grey", horiz=TRUE)
barplot_s(x, lowtrim=0.1, hightrim=0.8, colfun=heat.colors, coli=seq_along(x))
barplot_s(x, lowtrim=0.1, hightrim=0.8, colfun=heat.colors,
 coli=seq_along(x), trimlegend=FALSE, horiz=TRUE)
par(oldpar)



###################################################
### code chunk number 17: sRREbpbyself
###################################################
s  <- sumRprofRegressionExpl$by.self$self.time
names(s) <-  rownames(sumRprofRegressionExpl$by.self)
barplot_s(s, horiz=TRUE, col=rainbow(length(s)), las=1, main="self.time, by self")


###################################################
### code chunk number 18: tab:prSRREbt
###################################################
prxt(
	sumRprofRegressionExpl$by.total[ 
		sumRprofRegressionExpl$by.total$total.time>0.004,],
	caption="summaryRprof result: by.total, total time > 4ms", 
	label="tab:prSRREbt")


###################################################
### code chunk number 19: sRREbpbytotal
###################################################
s  <- sumRprofRegressionExpl$by.total$total.time
names(s) <-  rownames(sumRprofRegressionExpl$by.total)
barplot_s(s, horiz=TRUE, 
	col=rainbow(length(s)), las=1, main="total.time, by total")


###################################################
### code chunk number 20: readRprof
###################################################
sprof01xx<- readRprof("RprofsRegressionExpl.out", id="lm")


###################################################
### code chunk number 21: sprof01xx
###################################################
sprof01<- sprof01xx


###################################################
### code chunk number 22: sprofiling.Rnw:564-565
###################################################
str(sprof01$info)


###################################################
### code chunk number 23: sprofiling.Rnw:568-569
###################################################
sprof01$info$id <- "sprof01"


###################################################
### code chunk number 24: sprofiling.Rnw:573-576 (eval = FALSE)
###################################################
## sprof01lm<-sprof01
## save(sprof01lm, file="sprof01lm.RData")
## rm(sprof01lm)


###################################################
### code chunk number 25: sprofiling.Rnw:583-584 (eval = FALSE)
###################################################
## sprof01temp <- sampleRprof(runif(10000), runs=100)


###################################################
### code chunk number 26: sprofiling.Rnw:597-598
###################################################
strx(sprof01)


###################################################
### code chunk number 27: sprofiling.Rnw:600-601
###################################################
str(sprof01$nodes)


###################################################
### code chunk number 28: tab:prspbtself
###################################################
nodes <- sprof01$nodes[order(sprof01$nodes$self.time, 
decreasing=TRUE),]
rownames(nodes) <- NULL
prxt(nodes[nodes$self.time>4,1:7],
digits=c(0,0,0,2,0,2,0,2),
caption="sprof result: by.self, self time > 4ms", 
	label="tab:prspbtself")


###################################################
### code chunk number 29: tab:prspbt
###################################################
nodes <- sprof01$nodes[order(sprof01$nodes$total.time, 	
	decreasing=TRUE),]
rownames(nodes) <- NULL
prxt(nodes[nodes$total.time>4,1:7],
	digits=c(0,0,0,2,0,2,0,2),
	caption="sprof result: by.total, total time > 4ms", 
	label="tab:prspbt")


###################################################
### code chunk number 30: sREplotnodes01
###################################################
#8
oldpar <- par(mfrow=c(2,2))
plot_nodes(sprof01)
par(oldpar)


###################################################
### code chunk number 31: sREplotnodes01h
###################################################
#8
oldpar <- par(mfrow=c(2,2))
plot_nodes(sprof01, horiz=TRUE)
par(oldpar)


###################################################
### code chunk number 32: funnodeattrs
###################################################
nodeattrs <- function(sprof, 
	nodeattrs, # = c("default", "time", "runs"),
	maxcol=16, colpwr=0.5,
	maxlwd=7, lwdpow=0.5)
{
	#default
	if (missing(nodeattrs)) nodeattrs <- "default"
	icol <- rep(1,length(sprof$nodes$name))
	ilwd <- icol

	if (nodeattrs == "runs"){
		# may be skipped. if (is.null(sprof$nodes$avg_time)){
		nrl <- nodesrunlength(sprof, clean=FALSE)
		icol <-  rkindex(-nrl[,"avg_time"], 
				pwr=colpwr, maxindex=maxcol, ties.method="min")
		ilwd <-  rkindex(nrl[,"nr_runs"], 
			pwr=lwdpow, maxindex=maxlwd, ties.method="min")
	} else if (nodeattrs == "time"){
		icol <- rkindex(-sprof$nodes$self.time, 
				pwr=colpwr, maxindex=maxcol, ties.method="min")
		ilwd <-  rkindex(sprof$nodes$total.time, 
			pwr=lwdpow, maxindex=maxlwd, ties.method="min")
	}
	data.frame(icol=icol, ilwd=ilwd)
}


###################################################
### code chunk number 33: sprofiling.Rnw:751-754
###################################################
freqrank01 <- rkindex(-sprof01$nodes$total.time, 
	ties.method="random")
freqrankcol01 <- heat.colors(length(freqrank01))


###################################################
### code chunk number 34: sREplotnodes01freqcol
###################################################
#10
sprof01$nodes$icol <- freqrank01
oldpar <- par(mfrow=c(2,2))
plot_nodes(sprof01, col=freqrankcol01)
par(oldpar)


###################################################
### code chunk number 35: runs
###################################################
plot(sprof01$nodes$avg_time,sprof01$nodes$nr_runs)


###################################################
### code chunk number 36: sprofiling.Rnw:837-840
###################################################
nodekeyword0 <- function(node)
{
}


###################################################
### code chunk number 37: sprofiling.Rnw:842-845
###################################################
nodepackages <- nodepackage(sprof01$nodes$name)
names(nodepackages) <- sprof01$nodes$name
table(nodepackages)


###################################################
### code chunk number 38: sprofiling.Rnw:847-848
###################################################
sprof01$nodes$icol <-as.factor(nodepackages)


###################################################
### code chunk number 39: Rdplotnodes01
###################################################
oldpar <- par(mfrow=c(3,2))

if (require(RColorBrewer)) colpack <- 
	brewer.pal(length(levels(sprof01$nodes$icol)), "Paired") else 
	colpack <- rainbow(length(levels(sprof01$nodes$icol)))

plot_nodes(sprof01, which=1:6, col=colpack)
par(oldpar)


###################################################
### code chunk number 40: nodeclass
###################################################
nodeclass <- function(sprof)
{
	# use string list to define classes
	x_apply <- c("apply", "lapply", "vapply", "sapply")
	x_as <- c("as.list", "as.data.frame", "as.list.data.frame",
		"as.character", "as.list.default","as.name")
	# (Extend as you need it) and then use, as for example:
	ncl <- rep("x_nn", sprof$info$nrnodes)
	ncl[sprof$nodes$name %in% x_apply] <- "x_apply"
	ncl[sprof$nodes$name %in% x_as] <- "x_as"

	# or use assignments on the fly
	ncl[sprof$nodes$name %in% 
		c("eval",  "evalFunc",
			"try", "tryCatch", "tryCatchList", "tryCatchOne",
			"doTryCatch", "withVisible")
			] <- "x_eval"
	ncl[sprof$nodes$name %in% 
		c("model.frame", "model.matrix.default","model.frame.default",
		 " model.response", "model.matrix", "model.response", "coef")
			] <- "x_model"
	ncl[sprof$nodes$name %in% 
		c("[", "[.data.frame", "[[", "[[.data.frame") ] <- "x_acc"
	ncl[sprof$nodes$name %in% 
		c("lm", "lm.fit", "summary.lm")
			] <- "x_lm"
	ncl[sprof$nodes$name %in% 
		c("na.omit", "na.omit.data.frame")
			] <- "x_na"
		
	ncl[sprof$nodes$name =="<Anonymous>"] <- "x_Anon"
	ncl[sprof$nodes$name == "Sweave"] <- "x_Sweave"
	ncl[sprof$nodes$name %in% c( "summary" ,"summary.lm")] <-
		 "x_summary"

	return(as.factor(ncl))
}


###################################################
### code chunk number 41: sprofiling.Rnw:917-918
###################################################
sprof01$nodes$icol <-nodeclass(sprof01)


###################################################
### code chunk number 42: sprofiling.Rnw:922-927
###################################################
nodeclasscol <- c("red", "green", "blue", "yellow", 
	"cyan", "magenta", "purple", 
	"brown", "aquamarine", "pink", "violet")
	
# gold cyan4 aquamarine pink violet orchid hotpink salmon turquoise1


###################################################
### code chunk number 43: sREplotnodes01col
###################################################
#8 12
oldpar <- par(mfrow=c(3,2))
plot_nodes(sprof01, which=1:6, col=nodeclasscol)
par(oldpar)


###################################################
### code chunk number 44: sREplotnodes01colcb
###################################################
#8 12
oldpar <- par(mfrow=c(3,2))
plot_nodes(sprof01, which=1:6, las=2)
par(oldpar)


###################################################
### code chunk number 45: shownodes01rb
###################################################
#8 rainbow
sprof01$nodes$icol <- freqrank01; freqrankcol <- rainbow(62)
shownodes(sprof01, col=freqrankcol)


###################################################
### code chunk number 46: sprofiling.Rnw:1126-1130
###################################################
stacks_nodes <- list.as.matrix(sprof01$stacks$nodes)
nrnodes <- apply(stacks_nodes,1,function(x) {length(unique(x))})
cat("nr unique nodes per stack level\n")
nrnodes


###################################################
### code chunk number 47: nrunique
###################################################
plot(x=nrnodes, y= 1:length(nrnodes), 
	xlab="nr of unique nodes", ylab="stack level")
abline(h=2.5,col="green")
abline(h=12.5,col="green")


###################################################
### code chunk number 48: sprofiling.Rnw:1166-1167
###################################################
summary(sprof01$stacks$refcount)


###################################################
### code chunk number 49: sprofiling.Rnw:1175-1177
###################################################
summary(sprof01$stacks$refcount[sprof01$stacks$refcount>2])



###################################################
### code chunk number 50: tab:margin
###################################################
df <- data.frame(stack=sprof01$profiles$data, 
	count=sprof01$stacks$refcount[sprof01$profiles$data])
prxt(df, caption="Stacks by event: burn in/fade out",
	label="tab:margin",
	digits=c(0,0,0) )


###################################################
### code chunk number 51: sprofiling.Rnw:1254-1256
###################################################
cleanpackages()



###################################################
### code chunk number 52: sprofadjNEL026
###################################################

#6 sprofadjNEL02
library(graph)
sprof01adjNEL <- as(adjacency(sprof01),"graphNEL")

plot(sprof01adjNEL,  main="sprof01: graph layout example", 
	sub=sprof01$info$id,  
	attrs=list(node=list(cex=4, fontsize=40,  shape="ellipse")), 
	cex.main=2)
	
rm(sprof01adjNEL) # used for look-ahead example only


###################################################
### code chunk number 53: sprofiling.Rnw:1288-1289
###################################################
sprof02 <- sprof01; sprof02$info$id <- "sprof02: trimmed"


###################################################
### code chunk number 54: sprofiling.Rnw:1292-1295
###################################################
basetrim <- 13
sprof02$stacks$nodes <- sapply(sprof02$stacks$nodes, 
	function (x){if (length(x)> basetrim) x[-(1:basetrim)] })


###################################################
### code chunk number 55: sprofiling.Rnw:1299-1301
###################################################
sprof02$profiles$data[1:3] <- NA
sprof02$profiles$data[519:522] <- NA


###################################################
### code chunk number 56: sprofiling.Rnw:1305-1306
###################################################
strx(sprof02$info)


###################################################
### code chunk number 57: tab:sprof02info1
###################################################
nodes<- sprof02$nodes[,-8]; rownames(nodes) <- NULL
prxt(nodes,  
	caption="sprof02, before update",
	label="tab:sprof02info1",
	digits=c(0,0,0,2,0,2,0,2),
	zero.print=" . "
	)
	


###################################################
### code chunk number 58: sprof02Upd
###################################################
sprof02 <- updateRprof(sprof02)
sprof02$info$id <- "sprof02 updated"


###################################################
### code chunk number 59: sprofiling.Rnw:1326-1327
###################################################
strx(sprof02$info)


###################################################
### code chunk number 60: tab:sprof02info2
###################################################
nodes<- sprof02$nodes[,-8]; rownames(nodes) <- NULL
prxt(nodes, 
	caption="sprof02, after update", 
	label="tab:sprof02info2",
	#digits=c(0,0,0,2,0,2,0,2,0,2),
	digits=c(0,0,0,2,0,2,0,2),
	zero.print=" . "
)
	


###################################################
### code chunk number 61: Rdshownodes02
###################################################
#6 6
shownodes(sprof02)


###################################################
### code chunk number 62: sprofiling.Rnw:1365-1371
###################################################
stackm01 <- list.as.matrix(sprof01$stacks$nodes)
nr_uniquenodes01 <- apply(stackm01,1, function(x) { length(unique(x))})
names( nr_uniquenodes01)<- rownames(nr_uniquenodes01,FALSE,"L")
cat("Nr of unique nodes by level\n")
nr_uniquenodes01
rm(stackm01, nr_uniquenodes01)


###################################################
### code chunk number 63: sREimgprofilesTr
###################################################
rsprof01Tr <- trimstacks(sprof01, level=11)


###################################################
### code chunk number 64: sprofiling.Rnw:1408-1409
###################################################
roots_sprof(sprof01,stacks=rsprof01Tr)


###################################################
### code chunk number 65: sprofiling.Rnw:1415-1417
###################################################
rsprof02Tr <- trimstacks(sprof01, level=12)
roots_sprof(sprof01,stacks=rsprof02Tr)


###################################################
### code chunk number 66: sprofiling.Rnw:1482-1496
###################################################
addnode <- function(nodes, newnode, warn = options("warn"))
{
	i <- match(newnode, nodes$name, nomatch=0)
	if (i==0){
		nodes$name <- as.character(nodes$name)
		nodes <- rbind(nodes,NA)
		i <- length(nodes$name)
		nodes$name[i] <- newnode
		rownames(nodes) <- nodes$name
	if (as.logical(warn)) 
		message("addnode: node added. An updateRprof() may be necessary.")
	}
	return(nodes)
}


###################################################
### code chunk number 67: sprofiling.Rnw:1499-1504
###################################################
sprof03 <- sprof02; sprof03$info$id <- "sprof03: after surgery"
nodes <- addnode(sprof03$nodes, "<.getXlevels_[>", warn=FALSE)
nodes <- addnode(nodes, "<model_as.list>", warn=FALSE)
nodes <- addnode(nodes, "<model_matrix_as.list>", warn=FALSE)
sprof03$nodes <- nodes


###################################################
### code chunk number 68: sprofiling.Rnw:1512-1527
###################################################
xwhere <- function(oldseq, x){
	x <- unlist(x)
	firstpos <- 1; lastpos <-  length(x) -  length(oldseq) +1

	if (lastpos <1) return(0)
	l <- length(oldseq)-1
	while (firstpos <= lastpos) {
		if ( isTRUE (
		all.equal(oldseq , x[firstpos:(firstpos+l)], check.attributes=FALSE)
		) )
			{return(firstpos) }
		firstpos <- firstpos+1
		}
	return(0)
}


###################################################
### code chunk number 69: sprofiling.Rnw:1528-1538
###################################################
xreplace <- function(oldseq, newseq, x){
#! handle multiple replacements
	wh <- xwhere(oldseq,x)
	if (wh) {
	l <- length(oldseq)-1
	if (wh>1) x1 <- c(x[1:(wh-1)], newseq) else x1 <- newseq
	if (wh+l < length(x)) x <- c(x1, x[ -(1:(wh + l))] ) else x <- x1
	return(x)
	} else return(x)
}


###################################################
### code chunk number 70: sprofiling.Rnw:1539-1545
###################################################
nodenames2index <- function(names, sprof){
if (is.character(names)) 
		{ sapply(names, function(x) {match(x,sprof$nodes$name)})
		  #if (is.na(trimnode)) return(ts)
		} else names
}


###################################################
### code chunk number 71: sprofiling.Rnw:1549-1556
###################################################
xreplacenodes <- function(sprof, oldseq, newseq)
{
	if (is.character(oldseq)) oldseq <- nodenames2index(oldseq,sprof)
	if (is.character(newseq)) newseq <- nodenames2index(newseq,sprof)
	stacks <- sprof$stacks$nodes
	sapply(stacks, function(x){xreplace(oldseq, newseq, unlist(x))})
}


###################################################
### code chunk number 72: sprofiling.Rnw:1569-1580
###################################################
sprof03$stacks$nodes <- xreplacenodes(sprof03, 
	c(".getXlevels", "lapply", "["),
	"<.getXlevels_[>")
sprof03$stacks$nodes <- xreplacenodes(sprof03, 
	c("model.frame.default", "vapply", "as.list"),
	 "<model_as.list>")
sprof03$stacks$nodes <- xreplacenodes(sprof03, 
	c("model.matrix.default", "vapply", "as.list"),
	 "<model_matrix_as.list>")
	
#asfactormodel(sprof03$stacks$nodes, sprof03$nodes$name)


###################################################
### code chunk number 73: sprofiling.Rnw:1583-1589
###################################################
sprof03$nodes <- addnode(sprof03$nodes, 
	"<in_match>", warn=FALSE)

sprof03$stacks$nodes <- xreplacenodes(sprof03, 
	c("%in%", "match"),
	 "<in_match>")


###################################################
### code chunk number 74: sprofiling.Rnw:1592-1593
###################################################
sprof03 <- updateRprof(sprof03)


###################################################
### code chunk number 75: sprofiling.Rnw:1617-1619
###################################################
sprof04 <- readRprof("RprofsRegressionExpl03.out", id="sprof04")
sprof04$nodes$icol <-nodeclass(sprof04)


###################################################
### code chunk number 76: sREplotnodes04col
###################################################
#8 12
oldpar <- par(mfrow=c(3,2))
plot_nodes(sprof04, which=1:6, col=nodeclasscol)
par(oldpar)


###################################################
### code chunk number 77: sprofiling.Rnw:1657-1664
###################################################
profile_nodes <- profiles_matrix(sprof03)

profile_nodes_rle<- rrle(profile_nodes, collapseNA=FALSE) 
#!NA needs special case in run length handling.

strx(profile_nodes_rle, list.len=5)



###################################################
### code chunk number 78: profile_nodes_rlearray01
###################################################
profile_nodes_rlearray <- nodesprofile(sprof03)
strx(profile_nodes_rlearray)


###################################################
### code chunk number 79: nn_profile_nodes_rlearray
###################################################
# nn <- profile_nodes_rlearray["model.frame", , ]
# print.table(addmargins(nn), zero.print = ".")


###################################################
### code chunk number 80: tab:pramt4
###################################################
amt <- nodesrunlength(sprof03)
amt <- amt[1:20, 1:5]
prxt(amt,
	 caption=paste0("Marginal statistics on nodes by run length, ",
		 "sorted by total time used (table trimmed)"), 
	label="tab:pramt4", 
	digits=c(rep(0,dim(amt)[2]) ,2), 
	zero.print=" . ") #dim(amt)[2]-1, +1 for rownames


###################################################
### code chunk number 81: sprofiling.Rnw:1817-1818
###################################################
cleanpackages()


###################################################
### code chunk number 82: sprofadj01NEL
###################################################
#12 6
library(graph)
oldpar <- par(mfrow=c(1,2))
	plotviz(sprof01)
	plotviz(sprof02)
par(oldpar)


###################################################
### code chunk number 83: sprofadj01NELsurg
###################################################
#12 6
library(graph)
oldpar <- par(mfrow=c(1,2))
	plotviz(sprof03)
	plotviz(sprof04)
par(oldpar)


###################################################
### code chunk number 84: sprofiling.Rnw:1919-1920
###################################################
cleanpackages()


###################################################
### code chunk number 85: sprofiling.Rnw:1921-1923
###################################################
library(graph)
library(Rgraphviz)


###################################################
### code chunk number 86: sprofiling.Rnw:1927-1968
###################################################

as_graphNEL_sprof <- function(sprof, weight=TRUE){
	
	a04<-adjacency(sprof)
	
	rnames <- rownames(a04)
	
	if (!weight) {
	dimold <- dim(a04);a04 <- as.numeric(a04); dim(a04) <- dimold
	rownames(a04)<- rnames; colnames(a04)<- rnames;
	} #! define lwd first
	
	el04 <- edgedf(a04)
	el04$lwd  <- rkindex(el04$count, maxindex=7, ties.method="min")
	
	a04NEL <- as(a04,"graphNEL")
	nodeDataDefaults(a04NEL, "shape") <- "ellipse"
	nodeDataDefaults(a04NEL, "cex") <- 0.6
	nodeDataDefaults(a04NEL, "weigth") <- 1
	nodeDataDefaults(a04NEL, "fill") <- "green"
	nodeDataDefaults(a04NEL, "col") <- "yellow"
	
	a04NEL <- layoutGraph(a04NEL)
	
	nodeRenderInfo(a04NEL) <- list(shape="ellipse")
	nodeRenderInfo(a04NEL) <- list(cex=0.6, shape="ellipse")
	nodeRenderInfo(a04NEL) <- list(weight=1)
	#nodeRenderInfo(a04NEL) <- list(color="yellow")
	nodeRenderInfo(a04NEL) <- list(fill="yellow", col="blue")
	
	edgeDataDefaults(a04NEL,"lwd") <- 1
	edgeDataDefaults(a04NEL,"col") <- "grey"
	
	#nodeRenderInfo(a04NEL) <- list(weight=1)
	
	#edgeRenderInfo(a04NEL) <- list(lwd=el04$lwd)
	#edgeRenderInfo(a04NEL)$lwd <- el04$lwd
	for (i in 1:length(el04$lwd))
	{edgeRenderInfo(a04NEL)$lwd[i] <- el04$lwd[i]}
	a04NEL
}


###################################################
### code chunk number 87: plot_graphNEL_sprof
###################################################
# source('~/projects/rforge/sintro/pkg/sprof/R/pgn.R', chdir = TRUE)
plot_graphNEL_sprof <- function(sprof,
	layoutType = "dot", 
	nodeattrs, # = c("default", "time", "runs"),
	fill_list = NULL,
	lwd_list = NULL,
	srclegend="topleft",
	attrlegend="bottomright",
	main = NULL, sub = NULL,...)
{
	main1 <- deparse(substitute(sprof))
	xsubid <- NULL
	y<-NULL
	class1 <- NULL
	
	if (missing(nodeattrs)) nodeattrs <- "default"
	#nodeattrs <- match.arg(nodeattrs)
	# cat(nodeattrs)
	
	if (inherits(sprof, "sprof")) {
		class1 <- paste0("class orig: ", paste(class(sprof), collapse=" "))
		main1 <- sprof$info$id
		graphNEL <- 	
			as_graphNEL_sprof(sprof, weight=FALSE)
		y <- layoutType
	} else graphNEL <- sprof

	if (!inherits(graphNEL, "graphNEL")) warn("graphNEL data structure needed")

 	if (!is.null(sub)) sub <- as.character(sub) 
	
	main=paste0(main, "\n plot_graphNEL_sprof( ", 
		main1 , ", ", 
		layoutType, " )\n", xsubid)
		
# functions
	legattributes <- function(legnd=NULL, 
		nodecol=NULL,  nodelwd=NULL, 
		edgcol=NULL, edglwd="frequency")
	{
		# 
		llegnd<- legnd
		if (!is.null(nodecol)) {
			nodecol <- paste0("node col: ",nodecol, collapse="")
			if (is.null(llegnd))  llegnd <- nodecol else
				llegnd <- paste0(llegnd,"\n", nodecol,collapse="")
		}
		if (!is.null(nodelwd)) {
			nodelwd <- paste0("node lwd: ", nodelwd, collapse="")
			if (is.null(llegnd)) llegnd <- nodelwd else  
				llegnd <- paste0(llegnd,"\n", nodelwd, collapse="") 
		}
		if (!is.null(edgcol)) {
			edgcol <- paste0("edge col: ", edgcol, collapse="\n")
			if (is.null(llegnd)) llegnd <- edgcol else 
				llegnd <- paste0(llegnd,"\n", edgcol,collapse="")
		}
		if (!is.null(edglwd)) {
			edglwd <- paste0("edge lwd: ", edglwd, collapse="")
			if (is.null(llegnd)) llegnd <- edglwd else 
				llegnd <- paste0(llegnd, "\n",edglwd,collapse="")
		}
		if (!is.null(attrlegend)){
		if (!is.null(llegnd))		
		{	llegnd=paste0(llegnd,"\n")

		legend(attrlegend,
				legend= llegnd, 
				bg="#FFFFE0",
				seg.len=0,
				bty="n", 
				text.font=3)
			#bg="#0000"
			}
			}
	} # legattributes 
	
	graphNEL <- layoutGraph(graphNEL, layoutType=layoutType) 
	
	nodeDataDefaults(graphNEL, "shape") <- "ellipse"
	
	if (nodeattrs == "runs"){
		amt <- nodesrunlength(sprof, clean=FALSE)
	
		#sprof$nodes$self.time -> fill
		fill_list <- heat.colors(12)[
			rkindex(-amt[,"avg_time"], 
				pwr=0.5, maxindex=12, ties.method="min")]
		names(fill_list) <- sprof$nodes$name
	
		#sprof$nodes$total.time -> lwd
		lwd_list  <-  rkindex(amt[,"nr_runs"], 
			pwr=0.5, maxindex=7, ties.method="min")
		names(lwd_list) <- sprof$nodes$name
	
		#strx(nodeRenderInfo(graphNEL))
		nDD3 <- (nodeRenderInfo(graphNEL))
	
		nodeDataDefaults(graphNEL, "shape") <- "ellipse"
	
		nodeRenderInfo(graphNEL) <- list(lwd=lwd_list, 
			fill=fill_list, 
			col="#0000FF80", 
			shape="ellipse",
			weight=1)
		#strx(nodeRenderInfo(graphNEL))
	} else if (nodeattrs == "time"){
		# node attributes
		#sprof$nodes$self.time -> fill
		fill_list <- heat.colors(12)[
			rkindex(-sprof$nodes$self.time, 
				pwr=0.5, maxindex=12, ties.method="min")]
		names(fill_list) <- sprof$nodes$name
	
		#sprof$nodes$total.time -> lwd
		lwd_list  <-  rkindex(sprof$nodes$total.time, 
			pwr=0.5, maxindex=7, ties.method="min")
		names(lwd_list) <- sprof$nodes$name
	
		#strx(nodeRenderInfo(graphNEL))
		nDD3 <- (nodeRenderInfo(graphNEL))
	}   else {
		# graphNEL <- layoutGraph(graphNEL, layoutType=layoutType)
		fill_list <- NULL
		lwd_list <-  NULL
	}
	nodeDataDefaults(graphNEL, "shape") <- "ellipse"

	nodeRenderInfo(graphNEL) <- list(
		lwd=lwd_list, 
		fill=fill_list, 
		col="#0000FF80", 
		shape="ellipse",
		weight=1)
	
	# edge attributes
	# strx(edgeRenderInfo(graphNEL))
	nER01 <- edgeRenderInfo(graphNEL) 
	edgeRenderInfo(graphNEL) <- list(col= "#80808080")
	# strx(edgeRenderInfo(graphNEL))
	
	renderGraph(graphNEL)
	
	if (!is.null(srclegend))	
		legend(srclegend, 
			legend=c( class1, 
				paste0("class: ", 
					paste(class(graphNEL), collapse=" ")), 
				paste0("layout: ", layoutType)),
			bg="#FFFFE040",
			seg.len=0
			)#"lightyellow" =#FFFFE0
	
	 if (nodeattrs == "runs"){
		 title(main=paste0("nodes (run length)\n", main)	)	
		 legattributes(nodecol="avg run len, pwr=0.5",
				nodelwd="nr runs, pwr=0.5")
	} else if (nodeattrs == "time"){	
		title(main=paste0("nodes (time)\n", main))
		 legattributes( nodecol="self.time, pwr=0.5", 
			nodelwd="total.time, pwr=0.5")
	} else {
		title(main=main)
		legattributes( )
	}
	title( sub=sprof$info$id, col.sub=grey(0.5))
	invisible(graphNEL)
}# %:  plot_graphNEL_sprof(sprof03)


###################################################
### code chunk number 88: plot04NEL
###################################################
plot_graphNEL_sprof(sprof03)


###################################################
### code chunk number 89: sprofadj04NEL04
###################################################
#6 6
plot_graphNEL_sprof(sprof03, nodeattrs="time")


###################################################
### code chunk number 90: sprofiling.Rnw:2394-2395
###################################################
cleanpackages()


###################################################
### code chunk number 91: sprofadj04NEL02rle
###################################################
#6 6
library(Rgraphviz)

plot_graphNEL_sprof(sprof03, nodeattrs="runs")


###################################################
### code chunk number 92: sprofadj04NEL02rlenotit
###################################################
#6 6
library(Rgraphviz)

plot_graphNEL_sprof(sprof03, nodeattrs="runs", main="", sub="")


###################################################
### code chunk number 93: pgs01
###################################################
plot_graphNEL_sprof(sprof01, nodeattrs="runs")


###################################################
### code chunk number 94: pgs02
###################################################
plot_graphNEL_sprof(sprof02, nodeattrs="runs")


###################################################
### code chunk number 95: pgs03
###################################################
plot_graphNEL_sprof(sprof03, nodeattrs="runs")


###################################################
### code chunk number 96: pgs04
###################################################
plot_graphNEL_sprof(sprof03, nodeattrs="runs")


###################################################
### code chunk number 97: sprofiling.Rnw:2584-2585
###################################################
sprof <- sprof01


###################################################
### code chunk number 98: rpoprintnodes (eval = FALSE)
###################################################
## print_nodes(sprof)


###################################################
### code chunk number 99: rpoprintstacks (eval = FALSE)
###################################################
## print_stacks(sprof)


###################################################
### code chunk number 100: rpoprintprofiles (eval = FALSE)
###################################################
## print_profiles(sprof)


###################################################
### code chunk number 101: rposumnodes (eval = FALSE)
###################################################
## summary_nodes(sprof)


###################################################
### code chunk number 102: rposumstacks (eval = FALSE)
###################################################
## summary_stacks(sprof)


###################################################
### code chunk number 103: rposumprofiles (eval = FALSE)
###################################################
## summary_profiles(sprof)


###################################################
### code chunk number 104: rpoplotnodes (eval = FALSE)
###################################################
## plot_nodes(sprof)


###################################################
### code chunk number 105: rpoplotstacks (eval = FALSE)
###################################################
## plot_stacks(sprof)


###################################################
### code chunk number 106: rpoplotprofiles (eval = FALSE)
###################################################
## plot_profiles(sprof)


###################################################
### code chunk number 107: Rdrpoplotnodes
###################################################
oldpar <- par(mfrow=c(2,2))
plot_nodes(sprof)
par(oldpar)


###################################################
### code chunk number 108: Rdrpoplotstacks
###################################################
oldpar <- par(mfrow=c(1,2))
plot_stacks(sprof)
par(oldpar)


###################################################
### code chunk number 109: Rdrpoplotprofiles
###################################################
oldpar <- par(mfrow=c(2,2))
plot_profiles(sprof)
par(oldpar)


###################################################
### code chunk number 110: Rdplotsprof
###################################################
#12 16
oldpar <- par(mfrow=c(4,3))
plot.sprof(sprof03)
par(oldpar)


###################################################
### code chunk number 111: sprofiling.Rnw:2744-2745
###################################################
sprof <- sprof03


###################################################
### code chunk number 112: sprofiling.Rnw:2755-2757
###################################################
sprofadj <- adjacency(sprof)



###################################################
### code chunk number 113: sprofiling.Rnw:2762-2767
###################################################
sprofedgel <- edgedf(sprofadj)

sprofedgel$lwd  <- rkindex(sprofedgel$count, 
	maxindex=12, 
	ties.method="min")


###################################################
### code chunk number 114: sprofiling.Rnw:2777-2779
###################################################
cleanpackages()



###################################################
### code chunk number 115: sprofadjNEL02
###################################################
library(graph)
plotviz(sprof)


###################################################
### code chunk number 116: sprofiling.Rnw:2787-2788
###################################################
cleanpackages()


###################################################
### code chunk number 117: tab:spdemo
###################################################
spdemo <- demo(,"sprof")
prxt(spdemo$results[, c("Item","Title")])


###################################################
### code chunk number 118: sprofiling.Rnw:3073-3075
###################################################
cleanpackages()
#detach("package:sna")


###################################################
### code chunk number 119: sprofiling.Rnw:3079-3087
###################################################
library(network)
as_network_sprof <- function(sprof) {
	sprofadj <- adjacency(sprof)
	nwsprof <- as.network(sprofadj)
	network.vertex.names(nwsprof) <- 
		rownames(sprofadj) # not honoured by plot
	return(nwsprof)
}


###################################################
### code chunk number 120: plot_network_sprof
###################################################
plot_network_sprof <- function( nwsprof, 
	mode = "fruchtermanreingold", 
	main=NULL, label=NULL, sub=NULL,...) 
{ 	classnwsprof <- class(nwsprof)
	xid <- deparse(substitute(nwsprof))
	xsubid <- NULL
	
	if (inherits(nwsprof, "sprof")) {
		xsubid <- nwsprof$info$id
		nwsprof <-  as_network_sprof(nwsprof)
		}
	if (!is.null(label)) {
		warning("plot_network_sprof: explicit label supplied, but will use vertex names")
		if (!identical(label,network.vertex.names(nwsprof))) {
			print(strx(label))
			print(strx(network.vertex.names(nwsprof)))}
		}
	
	if (!is.null(sub)) sub <- as.character(sub)
		main <- 
			paste0(main, 
				"\n plot_network_sprof( ", xid ,", ", mode, " )\n", 
				xsubid)

	plot( nwsprof,
		label = network.vertex.names(nwsprof), 
		main= main, 
		mode = mode,
		edge.len=2,
		edge.col="#80808080",
		sub= sub,
		cex.main=1.5,...)
	
	legend("topleft", 
		legend=c(
			paste0("class: ",paste(class(classnwsprof), collapse=" ")), 
			paste0("mode: ",mode)),		
		bg="#FFFFE040",
		seg.len=0
		)
}


###################################################
### code chunk number 121: nwsprof03e
###################################################
#8 8 
edge.lwd<- sprofadj
edge.lwd[edge.lwd>0]<- rkindex(edge.lwd[edge.lwd>0], 
	maxindex=12, ties.method="min")
nwsprof03 <- as.network(sprofadj) # names is not imported 

plot_network_sprof(nwsprof03, label=rownames(sprofadj), 
 edge.lwd=edge.lwd)



###################################################
### code chunk number 122: nwsprof03circle
###################################################
plot_network_sprof(nwsprof03,
	 mode="circle", 
	 label=rownames(sprofadj), edge.lwd=edge.lwd)


###################################################
### code chunk number 123: nwsprof03kamadakawai
###################################################
plot_network_sprof(nwsprof03,
	mode="kamadakawai", 
	label=rownames(sprofadj), edge.lwd=edge.lwd)


###################################################
### code chunk number 124: sprofiling.Rnw:3163-3164
###################################################
cleanpackages()


###################################################
### code chunk number 125: sprofiling.Rnw:3171-3172
###################################################
library(Rgraphviz)


###################################################
### code chunk number 126: sprofadjvizdot
###################################################

# 6 6
plot_graphNEL_sprof(sprof03,  layoutType="dot", nodeattrs="runs")


###################################################
### code chunk number 127: sprofadjvizneato
###################################################
#6 6
plot_graphNEL_sprof(sprof03,  
	layoutType="neato", nodeattrs="runs", sub=sprof$info$id)


###################################################
### code chunk number 128: sprofadjviztwopi
###################################################
#6 6
plot_graphNEL_sprof(sprof03,  
	layoutType="twopi", nodeattrs="runs", sub=sprof$info$id)



###################################################
### code chunk number 129: sprofadjvizcirco6
###################################################
#6 6
plot_graphNEL_sprof(sprof03,  
	layoutType="circo", nodeattrs="runs", sub=sprof$info$id)



###################################################
### code chunk number 130: sprofadjvizfdp
###################################################
#6  6
plot_graphNEL_sprof(sprof03,  
	layoutType="fdp", nodeattrs="runs", sub=sprof$info$id)



###################################################
### code chunk number 131: sprofiling.Rnw:3254-3255
###################################################
toLatex(sessionInfo())


